@@ -31,5 +31,5 @@ struct matcher const matchers[] = {
 const char before_options[] =
 N_("PATTERN is, by default, a basic regular expression (BRE).\n");
 const char after_options[] =
-N_("`egrep' means `grep -E'.  `fgrep' means `grep -F'.\n\
-Direct invocation as either `egrep' or `fgrep' is deprecated.\n");
+N_("'egrep' means 'grep -E'.  'fgrep' means 'grep -F'.\n\
+Direct invocation as either 'egrep' or 'fgrep' is deprecated.\n");
