@@ -1,5 +1,5 @@
 /* grep.c - main driver file for grep.
-   Copyright (C) 1992, 1997-2002, 2004-2013 Free Software Foundation, Inc.
+   Copyright (C) 1992, 1997-2002, 2004-2014 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -44,6 +44,7 @@
 #include "progname.h"
 #include "propername.h"
 #include "quote.h"
+#include "safe-read.h"
 #include "version-etc.h"
 #include "xalloc.h"
 #include "xstrtol.h"
@@ -641,7 +642,7 @@ fillbuf (size_t save, struct stat const *st)
   readsize = buffer + bufalloc - readbuf;
   readsize -= readsize % pagesize;
 
-  fillsize = read (bufdesc, readbuf, readsize);
+  fillsize = safe_read (bufdesc, readbuf, readsize);
   if (fillsize < 0)
     fillsize = cc = 0;
   bufoffset += fillsize;
@@ -930,15 +931,18 @@ prline (char const *beg, char const *lim, int sep)
     line_color = match_color = NULL; /* Shouldn't be used.  */
 
   if ((only_matching && matching)
-      || (color_option  && (*line_color || *match_color)))
+      || (color_option && (*line_color || *match_color)))
     {
-      /* We already know that non-matching lines have no match (to colorize).  */
+      /* We already know that non-matching lines have no match (to colorize). */
       if (matching && (only_matching || *match_color))
         beg = print_line_middle (beg, lim, line_color, match_color);
 
-      /* FIXME: this test may be removable.  */
       if (!only_matching && *line_color)
-        beg = print_line_tail (beg, lim, line_color);
+        {
+          /* This code is exercised at least when grep is invoked like this:
+             echo k| GREP_COLORS='sl=01;32' src/grep k --color=always  */
+          beg = print_line_tail (beg, lim, line_color);
+        }
     }
 
   if (!only_matching && lim > beg)
@@ -1046,8 +1050,12 @@ prtext (char const *beg, char const *lim, intmax_t *nlinesp)
   used = 1;
 }
 
+/* Invoke the matcher, EXECUTE, on buffer BUF of SIZE bytes.  If there
+   is no match, return (size_t) -1.  Otherwise, set *MATCH_SIZE to the
+   length of the match and return the offset of the start of the match.  */
 static size_t
-do_execute (char const *buf, size_t size, size_t *match_size, char const *start_ptr)
+do_execute (char const *buf, size_t size, size_t *match_size,
+            char const *start_ptr)
 {
   size_t result;
   const char *line_next;
@@ -1070,7 +1078,8 @@ do_execute (char const *buf, size_t size, size_t *match_size, char const *start_
   for (line_next = buf; line_next < buf + size; )
     {
       const char *line_buf = line_next;
-      const char *line_end = memchr (line_buf, eolbyte, (buf + size) - line_buf);
+      const char *line_end = memchr (line_buf, eolbyte,
+                                     (buf + size) - line_buf);
       if (line_end == NULL)
         line_next = line_end = buf + size;
       else
@@ -1214,7 +1223,8 @@ grep (int fd, struct stat const *st)
             nlines += grepbuf (beg, lim);
           if (pending)
             prpending (lim);
-          if ((!outleft && !pending) || (nlines && done_on_match && !out_invert))
+          if ((!outleft && !pending)
+              || (nlines && done_on_match && !out_invert))
             goto finish_grep;
         }
 
@@ -2248,8 +2258,7 @@ main (int argc, char **argv)
     {
       /* A copy must be made in case of an xrealloc() or free() later.  */
       keycc = strlen (argv[optind]);
-      keys = xmalloc (keycc + 1);
-      strcpy (keys, argv[optind++]);
+      keys = xmemdup (argv[optind++], keycc + 1);
     }
   else
     usage (EXIT_TROUBLE);
