@@ -1,5 +1,5 @@
 /* pcresearch.c - searching subroutines using PCRE for grep.
-   Copyright 2000, 2007, 2009-2013 Free Software Foundation, Inc.
+   Copyright 2000, 2007, 2009-2014 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -36,11 +36,11 @@ static pcre *cre;
 /* Additional information about the pattern.  */
 static pcre_extra *extra;
 
-#ifdef PCRE_STUDY_JIT_COMPILE
+# ifdef PCRE_STUDY_JIT_COMPILE
 static pcre_jit_stack *jit_stack;
-#else
-#define PCRE_STUDY_JIT_COMPILE 0
-#endif
+# else
+#  define PCRE_STUDY_JIT_COMPILE 0
+# endif
 #endif
 
 void
@@ -60,10 +60,14 @@ Pcompile (char const *pattern, size_t size)
   char const *p;
   char const *pnul;
 
-#if defined HAVE_LANGINFO_CODESET
+# if defined HAVE_LANGINFO_CODESET
   if (STREQ (nl_langinfo (CODESET), "UTF-8"))
-    flags |= PCRE_UTF8;
-#endif
+    {
+      /* Enable PCRE's UTF-8 matching.  Note also the use of
+         PCRE_NO_UTF8_CHECK when calling pcre_extra, below.   */
+      flags |= PCRE_UTF8;
+    }
+# endif
 
   /* FIXME: Remove these restrictions.  */
   if (memchr (pattern, '\n', size))
@@ -111,7 +115,7 @@ Pcompile (char const *pattern, size_t size)
   if (ep)
     error (EXIT_TROUBLE, 0, "%s", ep);
 
-#if PCRE_STUDY_JIT_COMPILE
+# if PCRE_STUDY_JIT_COMPILE
   if (pcre_fullinfo (cre, extra, PCRE_INFO_JIT, &e))
     error (EXIT_TROUBLE, 0, _("internal error (should never happen)"));
 
@@ -122,12 +126,13 @@ Pcompile (char const *pattern, size_t size)
          than the interpreter, this should be enough in practice.  */
       jit_stack = pcre_jit_stack_alloc (32 * 1024, 512 * 1024);
       if (!jit_stack)
-        error (EXIT_TROUBLE, 0, _("cannot allocate memory for the JIT stack"));
+        error (EXIT_TROUBLE, 0,
+               _("failed to allocate memory for the PCRE JIT stack"));
       pcre_assign_jit_stack (extra, NULL, jit_stack);
     }
+# endif
   free (re);
-#endif
-#endif
+#endif /* HAVE_LIBPCRE */
 }
 
 size_t
@@ -153,6 +158,10 @@ Pexecute (char const *buf, size_t size, size_t *match_size,
        e == PCRE_ERROR_NOMATCH && line_next < buf + size;
        start_ofs -= line_next - line_buf)
     {
+      /* Disable the check that would make an invalid byte
+         seqence *in the input* trigger a failure.   */
+      int options = PCRE_NO_UTF8_CHECK;
+
       line_buf = line_next;
       line_end = memchr (line_buf, eolbyte, (buf + size) - line_buf);
       if (line_end == NULL)
@@ -167,7 +176,7 @@ Pexecute (char const *buf, size_t size, size_t *match_size,
         error (EXIT_TROUBLE, 0, _("exceeded PCRE's line length limit"));
 
       e = pcre_exec (cre, extra, line_buf, line_end - line_buf,
-                     start_ofs < 0 ? 0 : start_ofs, 0,
+                     start_ofs < 0 ? 0 : start_ofs, options,
                      sub, sizeof sub / sizeof *sub);
     }
 
@@ -185,9 +194,20 @@ Pexecute (char const *buf, size_t size, size_t *match_size,
           error (EXIT_TROUBLE, 0,
                  _("exceeded PCRE's backtracking limit"));
 
+        case PCRE_ERROR_BADUTF8:
+          error (EXIT_TROUBLE, 0,
+                 _("invalid UTF-8 byte sequence in input"));
+
         default:
-          abort ();
+          /* For now, we lump all remaining PCRE failures into this basket.
+             If anyone cares to provide sample grep usage that can trigger
+             particular PCRE errors, we can add to the list (above) of more
+             detailed diagnostics.  */
+          error (EXIT_TROUBLE, 0, _("internal PCRE error: %d"), e);
         }
+
+      /* NOTREACHED */
+      return -1;
     }
   else
     {
